from PyQt5 import QtWidgets, uic
from PyQt5.QtWidgets import QSlider
import numpy as np

import sys

class Frequency_Slider_Widget(QtWidgets.QWidget):

    def __init__(self):
        super(Frequency_Slider_Widget, self).__init__()
        uic.loadUi('frequency_slider_widget.ui', self)

        #Initialize the sliders
        self._init_gain_slider()
        self._init_center_frequency_slider()
        self._init_bandwidth_slider()
        

    def _init_gain_slider(self):
        """
        Initialize the gain slider from -15dB to 15dB.

        Parameters:
            N/A
        Returns:
            N/A
        """
        self.gain_slider.setMinimum(-15 * 1000) #-15dB
        self.gain_slider.setMaximum(15 * 1000)  #15dB
        self.gain_slider.setSingleStep(1)

        #Connect a change on the gain slider to update the gain_line_edit
        self.gain_slider.valueChanged.connect(self._update_gain_line_edit)
        self.gain_slider.setValue(0)
        self._update_gain_line_edit()

        #Set the ticks on the sliders
        self.gain_slider.setTickInterval(5000)
        self.gain_slider.setTickPosition(QSlider.TicksBothSides)

    def _update_gain_line_edit(self):
        """
        This is a callback function for when the gain slider is changed.
        It updates the gain (in dB) in the gain line edit.

        Parameters:
            N/A
        Returns:
            N/A
        """

        self.gain_dB  = self.gain_slider.value() / 1000.0 #Get the current value of the gain slider

        gain_string = "{:+.2f} dB".format(self.gain_dB)
        self.gain_line_edit.setText(gain_string)

        return

    def _init_center_frequency_slider(self):
        """
        Initialize the center frequency slider. This slider should give selectable
        frequency of 20 Hz to 20 kHz, which is the frequency range for hearable
        audio.

        Parameters:
            N/A
        Returns:
            N/A
        """
        self.center_frequency_slider.setMinimum((np.log10(20)) * 1000)
        self.center_frequency_slider.setMaximum((np.log10(20000)) * 1000)
        self.center_frequency_slider.setSingleStep(1)

        #Connect a change on the frequency slider to its line edit
        self.center_frequency_slider.valueChanged.connect(self._update_center_frequency_line_edit)
        self.center_frequency_slider.setValue(2801)
        self._update_center_frequency_line_edit()

        #Set the ticks on the sliders
        self.center_frequency_slider.setTickInterval(500)
        self.center_frequency_slider.setTickPosition(QSlider.TicksBothSides)

    def _update_center_frequency_line_edit(self):
        """
        Callback function for the center frequency slider to update the line edit.
        This display the current center frequency in the line edit.

        Parameters:
            N/A
        Returns:
            N/A
        """
        f_c = self.center_frequency_slider.value()
        self.center_frequency = 10**(f_c / 1000.0)

        if self.center_frequency < 1000:
            freq_string = "{:.2f} Hz".format(self.center_frequency)
        else:
            freq_string = "{:.2f} kHz".format(self.center_frequency / 1000.0)

        #write the string to the line edit to display
        self.center_frequency_line_edit.setText(freq_string)

    def _init_bandwidth_slider(self):
        """
        Initialize the bandwidth slider. The bandwidth range is from 20 hz to
        20000 Hz.

        Parameters:
            N/A
        Returns:
            N/A
        """
        self.bandwidth_slider.setMinimum((np.log10(20)) * 1000)
        self.bandwidth_slider.setMaximum((np.log10(20000)) * 1000)
        self.bandwidth_slider.setSingleStep(1)

        #Connect a change on the bandwidth slider to its line edit
        self.bandwidth_slider.valueChanged.connect(self._update_bandwidth_line_edit)
        self.bandwidth_slider.setValue(2801)
        self._update_bandwidth_line_edit()

        #Set the ticks on the sliders
        self.bandwidth_slider.setTickInterval(500)
        self.bandwidth_slider.setTickPosition(QSlider.TicksBothSides)

    def _update_bandwidth_line_edit(self):
        """
        Callback function for the bandwidth slider to update the line edit.
        This display the current bandwidth in the line edit.

        Parameters:
            N/A
        Returns:
            N/A
        """
        bandwidth = self.bandwidth_slider.value()
        self.bandwidth = 10**(bandwidth / 1000.0)

        if self.bandwidth < 1000:
            bandwidth_string = "{:.2f} Hz".format(self.bandwidth)
        else:
            bandwidth_string = "{:.2f} kHz".format(self.bandwidth / 1000.0)

        #write the string to the line edit to display
        self.bandwidth_line_edit.setText(bandwidth_string)


if __name__ == '__main__':
    app = QtWidgets.QApplication([])
    freq_slider = Frequency_Slider_Widget()
    freq_slider.show()
    sys.exit(app.exec_())
